@@ -15,6 +15,9 @@ import os
 import re
 import ssl
 import sys
+import urllib.parse
+import urllib.request
+from threading import Thread
 
 from pyVim.connect import Disconnect, SmartConnect
 from pyVmomi import vim, vmodl
@@ -101,6 +104,109 @@ def get_viserver_vm(conn, vm):
         raise exception.VMNotFound(vm=vm)
 
 
+def get_bootable_device_type(conn, boot_dev):
+    if isinstance(boot_dev, vim.vm.BootOptions.BootableFloppyDevice):
+        return "floppy"
+    elif isinstance(boot_dev, vim.vm.BootOptions.BootableDiskDevice):
+        return "disk"
+    elif isinstance(boot_dev, vim.vm.BootOptions.BootableCdromDevice):
+        return "cdrom"
+    elif isinstance(boot_dev, vim.vm.BootOptions.BootableEthernetDevice):
+        return "ethernet"
+
+
+def set_boot_device(conn, vm, device):
+    """Set boot device to specified device.
+
+    https://github.com/ansible-collections/vmware/blob/main/plugins/module_utils/vmware.py
+    """
+
+    boot_order_list = []
+    if device == "cdrom":
+        bootable_cdroms = [
+            dev
+            for dev in vm.config.hardware.device
+            if isinstance(dev, vim.vm.device.VirtualCdrom)
+        ]
+        if bootable_cdroms:
+            boot_order_list.append(vim.vm.BootOptions.BootableCdromDevice())
+    elif device == "disk":
+        bootable_disks = [
+            dev
+            for dev in vm.config.hardware.device
+            if isinstance(dev, vim.vm.device.VirtualDisk)
+        ]
+        if bootable_disks:
+            boot_order_list.extend(
+                [
+                    vim.vm.BootOptions.BootableDiskDevice(deviceKey=bootable_disk.key)
+                    for bootable_disk in bootable_disks
+                ]
+            )
+    elif device == "ethernet":
+        bootable_ethernets = [
+            dev
+            for dev in vm.config.hardware.device
+            if isinstance(dev, vim.vm.device.VirtualEthernetCard)
+        ]
+        if bootable_ethernets:
+            boot_order_list.extend(
+                [
+                    vim.vm.BootOptions.BootableEthernetDevice(
+                        deviceKey=bootable_ethernet.key
+                    )
+                    for bootable_ethernet in bootable_ethernets
+                ]
+            )
+    elif device == "floppy":
+        bootable_floppy = [
+            dev
+            for dev in vm.config.hardware.device
+            if isinstance(dev, vim.vm.device.VirtualFloppy)
+        ]
+        if bootable_floppy:
+            boot_order_list.append(vim.vm.BootOptions.BootableFloppyDevice())
+
+    kwargs = dict()
+    kwargs.update({"bootOrder": boot_order_list})
+
+    vm_conf = vim.vm.ConfigSpec()
+    vm_conf.bootOptions = vim.vm.BootOptions(**kwargs)
+    vm.ReconfigVM_Task(vm_conf)
+    return
+
+
+def send_nmi(conn, vm):
+    """Send NMI to specified VM.
+
+    https://github.com/vmware/pyvmomi/issues/726
+    """
+    context = None
+    if hasattr(ssl, "_create_unverified_context"):
+        context = ssl._create_unverified_context()
+
+    vmx_path = vm.config.files.vmPathName
+    for ds_url in vm.config.datastoreUrl:
+        vmx_path = vmx_path.replace("[%s] " % ds_url.name, "%s/" % ds_url.url)
+
+    url = "https://%s/cgi-bin/vm-support.cgi?manifests=%s&vm=%s" % (
+        vm.runtime.host.name,
+        urllib.parse.quote_plus("HungVM:Send_NMI_To_Guest"),
+        urllib.parse.quote_plus(vmx_path),
+    )
+
+    spec = vim.SessionManager.HttpServiceRequestSpec(method="httpGet", url=url)
+    ticket = conn.content.sessionManager.AcquireGenericServiceTicket(spec)
+    headers = {
+        "Cookie": "vmware_cgi_ticket=%s" % ticket.id,
+    }
+
+    req = urllib.request.Request(url, headers=headers)
+    Thread(
+        target=urllib.request.urlopen, args=(req,), kwargs={"context": context}
+    ).start()
+
+
 def check_viserver_connection_and_vm(vi, vm, vi_username=None, vi_password=None):
     with viserver_open(
         vi, readonly=True, vi_username=vi_username, vi_password=vi_password
